// Robot.h: interface for the robot class.
// Copyright: (c) 2008 Burkhard Wuensche
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ROBOT_H__CFA6100C_622B_478B_B2F1_E5DDF89FE34D__INCLUDED_)
#define AFX_ROBOT_H__CFA6100C_622B_478B_B2F1_E5DDF89FE34D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

const int NUM_LONGITUDINAL_VERTICES=10;
const int NUM_CIRCUMFERENTIALL_VERTICES=16;

class CRobot  
{
public:
	CRobot();
	virtual ~CRobot(){}
	void draw();
	void setTexture(GLuint i){ texName=i; _initTexture(); }
	void incrementPhi(double time){ phi+=0.3*(1+0.02*time); if (phi>360) phi-=360;}
	void decrementPhi(double time){ phi-=0.3*(1+0.02*time); if (phi<0) phi+=360;}
	void incrementTheta(double time){ theta+=0.3*(1+0.02*time); if (theta>thetaMax) theta=thetaMax;}
	void decrementTheta(double time){ theta-=0.3*(1+0.02*time); if (theta<thetaMin) theta=thetaMin;}
	// geometry
	static const double rimRadius;
	static const double basketRadius;
	static const double basketHeight;
	static const double armLength;
	static const double bottomArmWidth;
	static const double topArmWidth;
	void updateBasketPosition();
	float basketCentre[3];
private:
	// material properties
	GLfloat no_shininess[1];
	GLfloat mat_specular[4];
	GLfloat mat_ambient_and_diffuse[4];
	// animation parameters
	double thetaMin, thetaMax;
	double phi, theta;	// angles for rotating robot arms
	double timeLastStep;
	// attributes and methods for the basket at the tip of the robot arm
	void _drawBasket();
	float vertices[NUM_LONGITUDINAL_VERTICES][NUM_CIRCUMFERENTIALL_VERTICES][3];
	// texture parameters
	void _initTexture();
	float texture_coordinates[NUM_LONGITUDINAL_VERTICES][NUM_CIRCUMFERENTIALL_VERTICES][2];
	GLuint texName;
	GLubyte *texture;
	int textureWidth, textureHeight;
};

#endif // !defined(AFX_ROBOT_H__CFA6100C_622B_478B_B2F1_E5DDF89FE34D__INCLUDED_)
